/*
	< DanaSah is a chess engine winboard. >
    Copyright (C) <2013>  <Pedro Castro Elgarresta>
	E-mail <pecastro@msn.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Funciones para tener un static exchange evaluator */

/*Includes*/
#include "definiciones.h"
#include "variables.h"
#include "funciones.h"


/* Nos permite saber que algunos movimientos son malas capturas sin necesidad de hacer un caro SEE */
int malaCaptura(movimiento mcmov) {
	
	int n;
	
	/* Capturas "de igual a igual o de menos a ms" (tambin AxC) son buenas por definicin */
	if (valor_pieza2[pieza[mcmov.a]] >= valor_pieza2[pieza[mcmov.de]])
		return 0;

	/* Comprobamos si el oponente puede recapturar con un peon */
	if (color[mcmov.de] == BLANCO) {
		n = mailbox[mailbox64[mcmov.a] - 11];
		if (n != -1 && pieza[n] == PEON && color[n] == NEGRO) return 1;
		n = mailbox[mailbox64[mcmov.a] - 9];
		if (n != -1 && pieza[n] == PEON && color[n] == NEGRO) return 1;
	}
	else {
		n = mailbox[mailbox64[mcmov.a] + 9];
		if (n != -1 && pieza[n] == PEON && color[n] == BLANCO) return 1;
		n = mailbox[mailbox64[mcmov.a] + 11];
		if (n != -1 && pieza[n] == PEON && color[n] == BLANCO) return 1;
	}

	return 0;
}

/* Esta funcin calcula informacin de ataque-defensa para una casilla */
/* No tiene ataques o defensas por rayos x */
void calcula_atacantes(int casilla) {
	int i, n, ca_orig;
	register int ndir;
	int numB = num_atacantes[BLANCO], numN = num_atacantes[NEGRO];
	
	/* movimientos estilo alfil */
	for (i = 0; i < 4; i++) {
		ndir = offset[ALFIL][i];
		ca_orig = mailbox64[casilla]+ndir;
		n = mailbox[ca_orig];
		if (n == -1) continue;
		/* ataques por peones*/
		if (i >= 2 && pieza[n] == PEON && color[n] == BLANCO) {
			atacantes[BLANCO][numB].casilla = n;
			atacantes[BLANCO][numB].deslizante = 1;
			atacantes[BLANCO][numB].direccion = ndir;
			numB++;
		}
		else if (i <= 1 && pieza[n] == PEON && color[n] == NEGRO) {
			atacantes[NEGRO][numN].casilla = n;
			atacantes[NEGRO][numN].deslizante = 1;
			atacantes[NEGRO][numN].direccion = ndir;
			numN++;		
		}		
		/* el rey puede atacar solo desde una casilla contigua*/
		else if (pieza[n] == REY && color[n] == BLANCO) {
			atacantes[BLANCO][numB].casilla = n;
			numB++;
		}
		else if (pieza[n] == REY && color[n] == NEGRO) {
			atacantes[NEGRO][numN].casilla = n;
			numN++;
		}
		else {
			do {
				if ((pieza[n] == ALFIL || pieza[n] == DAMA) && color[n] == BLANCO) {
					atacantes[BLANCO][numB].casilla = n;
					atacantes[BLANCO][numB].deslizante = 1;
					atacantes[BLANCO][numB].direccion = ndir;
					numB++;
					break;
				}
				else if ((pieza[n] == ALFIL || pieza[n] == DAMA) && color[n] == NEGRO) {
					atacantes[NEGRO][numN].casilla = n;
					atacantes[NEGRO][numN].deslizante = 1;
					atacantes[NEGRO][numN].direccion = ndir;
					numN++;
					break;
				}
				else if (color[n] == !VACIO) break;
				ca_orig += ndir;
				n = mailbox[ca_orig];
			} while (n != -1);
		}
	}
	
	/* movimientos estilo caballo */
	for (i = 0; i < 8; i++) {
		n = mailbox[mailbox64[casilla]+offset[CABALLO][i]];
		if (n == -1) continue;
		if (pieza[n] == CABALLO && color[n] == BLANCO) {
			atacantes[BLANCO][numB].casilla = n;
			numB++;
		}
		else if (pieza[n] == CABALLO && color[n] == NEGRO) {
			atacantes[NEGRO][numN].casilla = n;
			numN++;	
		}
	}
	
	/* movimientos estilo torre */
	for (i = 0; i < 4; i++) {
		ndir = offset[TORRE][i];
		ca_orig = mailbox64[casilla]+ndir;
		n = mailbox[ca_orig];
		if (n == -1) continue;
		/* el rey puede atacar solo desde una casilla contigua*/
		if (pieza[n] == REY && color[n] == BLANCO) {
			atacantes[BLANCO][numB].casilla = n;
			numB++;
		}
		else if (pieza[n] == REY && color[n] == NEGRO) {
			atacantes[NEGRO][numN].casilla = n;
			numN++;		
		}
		else {
			do {
				if ((pieza[n] == TORRE || pieza[n] == DAMA) && color[n] == BLANCO) {
					atacantes[BLANCO][numB].casilla = n;
					atacantes[BLANCO][numB].deslizante = 2;
					atacantes[BLANCO][numB].direccion = ndir;
					numB++;
					break;
				}
				else if ((pieza[n] == TORRE || pieza[n] == DAMA) && color[n] == NEGRO) {
					atacantes[NEGRO][numN].casilla = n;
					atacantes[NEGRO][numN].deslizante = 2;
					atacantes[NEGRO][numN].direccion = ndir;
					numN++;
					break;						
				}
				else if (color[n] == !VACIO) break;
				ca_orig += ndir;
				n = mailbox[ca_orig];
			} while (n != -1);
		}
	}
	
	num_atacantes[BLANCO] = numB;
	num_atacantes[NEGRO] = numN;
}

/* SEE Static Exchange Evaluator */
int see(int turno, int casilla, int de) {
	int i, n, d;
	int valor_del_menor, indice_del_menor, casilla_del_menor, valor_temp;
	int valor[32];
	int origpieza, origcolor;
	int ganancia;
	int ca_orig;
		
	/* salida rpida */
	if (valor_pieza2[pieza[casilla]] >= valor_pieza2[pieza[de]]) return 0;
	
	/* ponemos a 0 algunas variables */	
	for (i = 0; i < 16; i++){
		atacantes[BLANCO][i].casilla = atacantes[NEGRO][i].casilla = 0;
		atacantes[BLANCO][i].deslizante = atacantes[NEGRO][i].deslizante = 0;
		atacantes[BLANCO][i].direccion = atacantes[NEGRO][i].direccion = 0;		
	}
	for (i = 0; i < 32; i++) valor[i] = 0;
	num_atacantes[BLANCO] = 0;
	num_atacantes[NEGRO] = 0;
	d = 0;
	
	/* eliminamos el capturador original del tablero */
	origpieza = pieza[de];
	origcolor = color[de];
	pieza[de] = VACIO;
	color[de] = VACIO;
	
	/* calcula todos los atacantes para la casilla */
	calcula_atacantes(casilla);
	
	/* inicialmente ganamos la pieza que capturamos */
	valor[d] = valor_pieza2[pieza[casilla]];
	
	/* es una captura libre? */
	if (num_atacantes[1-turno] == 0) {
		pieza[de] = origpieza;
		color[de] = origcolor;
		return valor[d];
	}
	
	/* iniciamos la primera recaptura */
	d++;
	ganancia = valor_pieza2[origpieza];
	turno ^= 1;
	
	/* continamos mientras haya atacantes */
	while(num_atacantes[turno])
	{	
		valor_del_menor = valor_pieza2[REY];
		indice_del_menor = 0;
		casilla_del_menor = 0;
		
		/*Buscamos el atacante de menor valor */
		for(i = 0; i < num_atacantes[turno]; i++) {	
			valor_temp = valor_pieza2[pieza[atacantes[turno][i].casilla]];
			if (valor_temp < valor_del_menor) {
				valor_del_menor = valor_temp;
				indice_del_menor = i;
			}
		}
		
		/* sumamos piezas que atacan y defienden mediante rayos x */
		if (atacantes[turno][indice_del_menor].deslizante == 1){
			int ndir = atacantes[turno][indice_del_menor].direccion;
			int principio = atacantes[turno][indice_del_menor].casilla;
			ca_orig = mailbox64[principio]+ndir;
			n = mailbox[ca_orig];
			while (n != -1) {
				if ((pieza[n] == ALFIL || pieza[n] == DAMA) && color[n] == BLANCO) {
					atacantes[BLANCO][num_atacantes[BLANCO]].casilla = n;
					atacantes[BLANCO][num_atacantes[BLANCO]].deslizante = 1;
					atacantes[BLANCO][num_atacantes[BLANCO]].direccion = ndir;
					num_atacantes[BLANCO]++;
					break;
				}
				else if ((pieza[n] == ALFIL || pieza[n] == DAMA) && color[n] == NEGRO) {
					atacantes[NEGRO][num_atacantes[NEGRO]].casilla = n;
					atacantes[NEGRO][num_atacantes[NEGRO]].deslizante = 1;
					atacantes[NEGRO][num_atacantes[NEGRO]].direccion = ndir;
					num_atacantes[NEGRO]++;
					break;
				}
				else if (color[n] == !VACIO) break;
				ca_orig += ndir;
				n = mailbox[ca_orig];
			}
		}
		else if (atacantes[turno][indice_del_menor].deslizante == 2){
			int ndir = atacantes[turno][indice_del_menor].direccion;
			int principio = atacantes[turno][indice_del_menor].casilla;
			ca_orig = mailbox64[principio]+ndir;
			n = mailbox[ca_orig];
			while (n != -1) {
				if ((pieza[n] == TORRE || pieza[n] == DAMA) && color[n] == BLANCO) {
					atacantes[BLANCO][num_atacantes[BLANCO]].casilla = n;
					atacantes[BLANCO][num_atacantes[BLANCO]].deslizante = 2;
					atacantes[BLANCO][num_atacantes[BLANCO]].direccion = ndir;
					num_atacantes[BLANCO]++;
					break;
				}
				else if ((pieza[n] == TORRE || pieza[n] == DAMA) && color[n] == NEGRO) {
					atacantes[NEGRO][num_atacantes[NEGRO]].casilla = n;
					atacantes[NEGRO][num_atacantes[NEGRO]].deslizante = 2;
					atacantes[NEGRO][num_atacantes[NEGRO]].direccion = ndir;
					num_atacantes[NEGRO]++;
					break;
				}
				else if (color[n] == !VACIO) break;
				ca_orig += ndir;
				n = mailbox[ca_orig];
			}
		}
		/*Intercambiamos la posicion de la ficha que hemos utilizado, con la
		que hemos comparado, para no volver a utilizarla */
		casilla_del_menor = atacantes[turno][indice_del_menor].casilla;
		atacantes[turno][indice_del_menor] = atacantes[turno][--num_atacantes[turno]];
		atacantes[turno][num_atacantes[turno]].casilla = casilla_del_menor;
			
		valor[d] = -valor[d-1] + ganancia;
		/*if (-MIN(-valor[d], valor[d-1]) < 0) break;*/
		
		/* cambiamos el turno */
		d++;
		ganancia = valor_del_menor;
		turno ^= 1;
	}
	
	while(--d)
		valor[d-1] = MIN(-valor[d], valor[d-1]);
	
	/* restauramos capturador */
	pieza[de] = origpieza;
	color[de] = origcolor;
	
	return (valor[0]);
}

int see2(int turno, int casilla, int de) {
	int i, n, d;
	int valor_del_menor, indice_del_menor, casilla_del_menor, valor_temp;
	int valor[32];
	int origpieza, origcolor;
	int ganancia;
	int ca_orig;
		
	/* salida rpida */
	if (valor_pieza2[pieza[casilla]] >= valor_pieza2[pieza[de]]) return 0;
	
	/* ponemos a 0 algunas variables */	
	for (i = 0; i < 16; i++){
		atacantes[BLANCO][i].casilla = atacantes[NEGRO][i].casilla = 0;
		atacantes[BLANCO][i].deslizante = atacantes[NEGRO][i].deslizante = 0;
		atacantes[BLANCO][i].direccion = atacantes[NEGRO][i].direccion = 0;		
	}
	for (i = 0; i < 32; i++) valor[i] = 0;
	num_atacantes[BLANCO] = 0;
	num_atacantes[NEGRO] = 0;
	d = 0;
	
	/* eliminamos el capturador original del tablero */
	origpieza = pieza[de];
	origcolor = color[de];
	pieza[de] = VACIO;
	color[de] = VACIO;
	
	/* calcula todos los atacantes para la casilla */
	calcula_atacantes(casilla);
	
	/* inicialmente ganamos la pieza que capturamos */
	valor[d] = valor_pieza2[pieza[casilla]];
	
	/* es una captura libre? */
	if (num_atacantes[1-turno] == 0) {
		pieza[de] = origpieza;
		color[de] = origcolor;
		return valor[d];
	}
	
	/* iniciamos la primera recaptura */
	d++;
	ganancia = valor_pieza2[origpieza];
	turno ^= 1;
	
	/* continamos mientras haya atacantes */
	while(num_atacantes[turno])
	{	
		valor_del_menor = valor_pieza2[REY];
		indice_del_menor = 0;
		casilla_del_menor = 0;
		
		/*Buscamos el atacante de menor valor */
		for(i = 0; i < num_atacantes[turno]; i++) {	
			valor_temp = valor_pieza2[pieza[atacantes[turno][i].casilla]];
			if (valor_temp < valor_del_menor) {
				valor_del_menor = valor_temp;
				indice_del_menor = i;
			}
		}
		
		/* sumamos piezas que atacan y defienden mediante rayos x */
		if (atacantes[turno][indice_del_menor].deslizante == 1){
			int ndir = atacantes[turno][indice_del_menor].direccion;
			int principio = atacantes[turno][indice_del_menor].casilla;
			ca_orig = mailbox64[principio]+ndir;
			n = mailbox[ca_orig];
			while (n != -1) {
				if ((pieza[n] == ALFIL || pieza[n] == DAMA) && color[n] == BLANCO) {
					atacantes[BLANCO][num_atacantes[BLANCO]].casilla = n;
					atacantes[BLANCO][num_atacantes[BLANCO]].deslizante = 1;
					atacantes[BLANCO][num_atacantes[BLANCO]].direccion = ndir;
					num_atacantes[BLANCO]++;
					break;
				}
				else if ((pieza[n] == ALFIL || pieza[n] == DAMA) && color[n] == NEGRO) {
					atacantes[NEGRO][num_atacantes[NEGRO]].casilla = n;
					atacantes[NEGRO][num_atacantes[NEGRO]].deslizante = 1;
					atacantes[NEGRO][num_atacantes[NEGRO]].direccion = ndir;
					num_atacantes[NEGRO]++;
					break;
				}
				else if (color[n] == !VACIO) break;
				ca_orig += ndir;
				n = mailbox[ca_orig];
			}
		}
		else if (atacantes[turno][indice_del_menor].deslizante == 2){
			int ndir = atacantes[turno][indice_del_menor].direccion;
			int principio = atacantes[turno][indice_del_menor].casilla;
			ca_orig = mailbox64[principio]+ndir;
			n = mailbox[ca_orig];
			while (n != -1) {
				if ((pieza[n] == TORRE || pieza[n] == DAMA) && color[n] == BLANCO) {
					atacantes[BLANCO][num_atacantes[BLANCO]].casilla = n;
					atacantes[BLANCO][num_atacantes[BLANCO]].deslizante = 2;
					atacantes[BLANCO][num_atacantes[BLANCO]].direccion = ndir;
					num_atacantes[BLANCO]++;
					break;
				}
				else if ((pieza[n] == TORRE || pieza[n] == DAMA) && color[n] == NEGRO) {
					atacantes[NEGRO][num_atacantes[NEGRO]].casilla = n;
					atacantes[NEGRO][num_atacantes[NEGRO]].deslizante = 2;
					atacantes[NEGRO][num_atacantes[NEGRO]].direccion = ndir;
					num_atacantes[NEGRO]++;
					break;
				}
				else if (color[n] == !VACIO) break;
				ca_orig += ndir;
				n = mailbox[ca_orig];
			}
		}
		/*Intercambiamos la posicion de la ficha que hemos utilizado, con la
		que hemos comparado, para no volver a utilizarla */
		casilla_del_menor = atacantes[turno][indice_del_menor].casilla;
		atacantes[turno][indice_del_menor] = atacantes[turno][--num_atacantes[turno]];
		atacantes[turno][num_atacantes[turno]].casilla = casilla_del_menor;
			
		valor[d] = -valor[d-1] + ganancia;
		if (-MIN(-valor[d], valor[d-1]) < 0) break;
		
		/* cambiamos el turno */
		d++;
		ganancia = valor_del_menor;
		turno ^= 1;
	}
	
	while(--d)
		valor[d-1] = MIN(-valor[d], valor[d-1]);
	
	/* restauramos capturador */
	pieza[de] = origpieza;
	color[de] = origcolor;
	
	return (valor[0]);
}
